@@ -55,55 +55,70 @@ class EntryType(Enum):
 
 # A reference to a ticket (e.g. #123 or libats#12), usually at the end of each
 # entry
-TicketRef = namedtuple('TicketRef', [
-    'number', # type: int
-    'repo',   # type: Optional[str]
-])
+TicketRef = namedtuple(
+    'TicketRef',
+    [
+        'number',  # type: int
+        'repo',  # type: Optional[str]
+    ])
 
 # Represents an entry in a change log. The associated tickets may be an empty list
-EntryLog = namedtuple('EntryLog', [
-    'internal',  # type: bool
-    'text',      # type: str
-    'tickets',   # type: List[TicketRef]
+EntryLog = namedtuple(
+    'EntryLog',
+    [
+        'internal',  # type: bool
+        'text',  # type: str
+        'tickets',  # type: List[TicketRef]
     ])
 
 # A category of change logs for a given version
-CategoryLog = namedtuple('CategoryLog', [
-    'entry_type',  # type: EntryType
-    'logs',        # type: List[EntryLog]
+CategoryLog = namedtuple(
+    'CategoryLog',
+    [
+        'entry_type',  # type: EntryType
+        'logs',  # type: List[EntryLog]
     ])
 
 # A version number that follows semantic versioning
-VersionNumber = namedtuple('VersionNumber', [
-    'unreleased',  # type: bool
-    'major',       # type: int
-    'minor',       # type: int
-    'patch',       # type: int
-    'prerelease',  # type: str
-    'build',       # type: str
+VersionNumber = namedtuple(
+    'VersionNumber',
+    [
+        'unreleased',  # type: bool
+        'major',  # type: int
+        'minor',  # type: int
+        'patch',  # type: int
+        'prerelease',  # type: str
+        'build',  # type: str
     ])
 
 # Represents the identity of a version log (version number,
 # release date, etc.)
-VersionInfo = namedtuple('VersionInfo', [
-    'version',  # type: VersionNumber
-    'date',     # type: datetime.date
-    'yanked',   # type: bool
+VersionInfo = namedtuple(
+    'VersionInfo',
+    [
+        'version',  # type: VersionNumber
+        'date',  # type: datetime.date
+        'yanked',  # type: bool
     ])
 
 # List of logs for version numbers
-VersionLog = namedtuple('VersionLog', [
-    'info',  # type: VersionInfo
-    'logs',  # type: List[CategoryLog]
+VersionLog = namedtuple(
+    'VersionLog',
+    [
+        'info',  # type: VersionInfo
+        'logs',  # type: List[CategoryLog]
     ])
 
 # A complete change log file
-ChangeLog = namedtuple('ChangeLog', [
-    'title',  # type: str
-    'intro',  # type: str
-    'logs',   # type: List[VersionLog]
+ChangeLog = namedtuple(
+    'ChangeLog',
+    [
+        'title',  # type: str
+        'intro',  # type: str
+        'logs',  # type: List[VersionLog]
     ])
 
+
 class ChangeLogRuleVisitor(NodeVisitor):
     '''Parses keepachangelog.org v1.0.0 change logs'''
     grammar = Grammar(r"""
@@ -192,9 +207,7 @@ class ChangeLogRuleVisitor(NodeVisitor):
         return int(node.text)
 
     def visit_sv2_version_core(self, node, childs):
-        return {'major': childs[0],
-                'minor': childs[2],
-                'patch': childs[4]}
+        return {'major': childs[0], 'minor': childs[2], 'patch': childs[4]}
 
     def visit_sv2_semver_core_prerelease_build(self, node, childs):
         result = {}
@@ -291,8 +304,8 @@ class ChangeLogRuleVisitor(NodeVisitor):
         return False
 
     def visit_unreleased_h2(self, _, __):
-        return VersionInfo(
-            VersionNumber(True, 0, 0, 0, "", ""), datetime.date.today(), False)
+        return VersionInfo(VersionNumber(True, 0, 0, 0, "", ""),
+                           datetime.date.today(), False)
 
     def visit_date(self, _, childs):
         return datetime.date(childs[0], childs[2], childs[4])
@@ -323,11 +336,13 @@ class ChangeLogRuleVisitor(NodeVisitor):
     def visit_changelog(self, _, childs):
         return ChangeLog(childs[0], childs[1], childs[2])
 
+
 def ticketref_text(ticketref: TicketRef):
     if ticketref.repo is not None:
         return "{}#{}".format(ticketref.repo, ticketref.number)
     return "#{}".format(ticketref.number)
 
+
 def entry_text(entry: EntryLog, output_internal: bool):
     text = entry.text
     if output_internal and len(entry.tickets):
@@ -336,6 +351,7 @@ def entry_text(entry: EntryLog, output_internal: bool):
         text += "]"
     return text
 
+
 def version_log_empty(vlog, output_internal=False):
     '''A version log is considered empty if it would not output any entry'''
 
@@ -396,6 +412,38 @@ def changelogs_merge(logs):
 def changelog_version(changelog):
     return changelog.logs[0].info.version
 
+'''Returns true if left and right are two released version numbers and if left
+is higher than right'''
+def is_more_than(left, right):
+    if left.unreleased or right.unreleased:
+        return False
+    lv = (left.major, left.minor, left.patch)
+    rv = (right.major, right.minor, right.patch)
+    return lv > rv
+
+def check_date_and_version_ordering(changelog):
+    # Verifies that dates and versions are in descending order in the change
+    # log
+    logs = changelog.logs
+    if not len(logs):
+        return  # Empty change log
+    curver = logs[0].info.version
+    curdate = logs[0].info.date
+    for i in range(1, len(logs)):
+        log = logs[i]
+        ver = log.info.version
+        date = log.info.date
+        if is_more_than(ver, curver):
+            raise ValueError(
+                "Versions in change log are not descending. Log index {} has version {} but previous was {}"
+                .format(i, ver, curver))
+        curver = ver
+        if date > curdate:
+            raise ValueError(
+                "Dates are not descending. Log index {} has date {} but previous was {}"
+                .format(i, date, curdate))
+        curdate = date
+
 
 def version_string(version, show_unreleased=True):
     if show_unreleased and version.unreleased:
@@ -432,13 +480,13 @@ def changelog_to_html(changelog, output_internal=False):
             for entry in cat.logs:
                 if entry.internal and not output_internal:
                     continue
-                out = out + "    <li>{}</li>\n".format(entry_text(entry, output_internal))
+                out = out + "    <li>{}</li>\n".format(
+                    entry_text(entry, output_internal))
             out = out + "</ul>\n"
     return out
 
 
-def changelog_to_md(changelog,
-                    output_internal=False):
+def changelog_to_md(changelog, output_internal=False):
     out = "# {}\n\n{}\n".format(changelog.title, changelog.intro)
 
     for versionlog in changelog.logs:
@@ -448,7 +496,8 @@ def changelog_to_md(changelog,
             out = out + "\n## [Unreleased]"
         else:
             out = out + "\n## [{}]".format(version_string(ver))
-            out = out + " - " + "{:04d}-{:02d}-{:02d}".format(dat.year, dat.month, dat.day)
+            out = out + " - " + "{:04d}-{:02d}-{:02d}".format(
+                dat.year, dat.month, dat.day)
 
         if version_log_empty(versionlog, output_internal):
             out = out + "\n- No user-visible change\n"
@@ -459,20 +508,22 @@ def changelog_to_md(changelog,
             for entry in cat.logs:
                 if entry.internal:
                     if output_internal:
-                        out = out + "\n- [Internal] {}".format(entry_text(entry, output_internal))
+                        out = out + "\n- [Internal] {}".format(
+                            entry_text(entry, output_internal))
                 else:
-                    out = out + "\n- {}".format(entry_text(entry, output_internal))
+                    out = out + "\n- {}".format(
+                        entry_text(entry, output_internal))
             out = out + "\n"
     return out
 
 
-def changelog_to_deb(changelog, package_name,
-                     output_internal=False):
+def changelog_to_deb(changelog, package_name, output_internal=False):
     out = ""
     for versionlog in changelog.logs:
         ver = versionlog.info.version
         dat = versionlog.info.date
-        out = out + "{} ({}) UNRELEASED; urgency=medium\n\n".format(package_name, version_string(ver, False))
+        out = out + "{} ({}) UNRELEASED; urgency=medium\n\n".format(
+            package_name, version_string(ver, False))
 
         if version_log_empty(versionlog, output_internal):
             out = out + "  * No user-visible change\n"
@@ -481,20 +532,23 @@ def changelog_to_deb(changelog, package_name,
                 for entry in cat.logs:
                     if entry.internal and not output_internal:
                         continue
-                    out = out + "  * {}: {}\n".format(cat.entry_type.name, entry_text(entry, output_internal))
+                    out = out + "  * {}: {}\n".format(
+                        cat.entry_type.name, entry_text(
+                            entry, output_internal))
 
-        out = out + "\n -- AlazarTech <support@alazartech.com>  {} 12:12:00 -0400\n\n".format(dat.strftime('%a, %d %b %Y'))
+        out = out + "\n -- AlazarTech <support@alazartech.com>  {} 12:12:00 -0400\n\n".format(
+            dat.strftime('%a, %d %b %Y'))
     return out
 
 
-def expand_spec(changelog, input_text,
-                output_internal=False):
+def expand_spec(changelog, input_text, output_internal=False):
     out = input_text.replace("@VERSION@",
                              version_string(changelog_version(changelog)))
     for versionlog in changelog.logs:
         dat = versionlog.info.date
         ver = versionlog.info.version
-        out = out + "* {} AlazarTech <support@alazartech.com>\n".format(dat.strftime('%a %b %d %Y'))
+        out = out + "* {} AlazarTech <support@alazartech.com>\n".format(
+            dat.strftime('%a %b %d %Y'))
         out = out + "- Version {}\n".format(version_string(ver))
 
         if version_log_empty(versionlog, output_internal):
@@ -505,10 +559,12 @@ def expand_spec(changelog, input_text,
             for entry in cat.logs:
                 if entry.internal and not output_internal:
                     continue
-                out = out + "- {}: {}\n".format(cat.entry_type.name, entry_text(entry, output_internal))
+                out = out + "- {}: {}\n".format(
+                    cat.entry_type.name, entry_text(entry, output_internal))
 
     return out
 
+
 def parse_changelog(filename):
     visitor = ChangeLogRuleVisitor()
     with open(filename, 'r') as clfile:
@@ -525,6 +581,7 @@ def main():
             inputs.append(visitor.parse(clfile.read()))
 
     changelog = changelogs_merge(inputs)
+    check_date_and_version_ordering(changelog)
 
     if arguments['version']:
         ver = version_string(changelog_version(changelog))
