@@ -25,6 +25,8 @@ Arguments:
 
 '''
 
+import os
+
 import json
 from collections import namedtuple
 
@@ -66,31 +68,34 @@ PRODUCTS = {
     "ATS9120": '1028',
     "ATS9130": '1030',
     "ATS9146": '1039',
+    "ATS9182": '3007',
     "ATS9350": '4',
     "ATS9351": '5',
     "ATS9352": '1036',
     "ATS9353": '1041',
     "ATS9360": '2',
+    "ATS9364": '3004',
     "ATS9371": '1027',
     "ATS9373": '1',
+    "ATS9376": '3006',
+    "ATS9380": '3005',
     "ATS9416": '1016',
+    "ATS9428": '3008',
+    "ATS9437": '3002',
     "ATS9440": '9',
+    "ATS9442": '1048',
+    "ATS9453": '1037',
     "ATS9462": '8',
+    "ATS9470": '1043',
+    "ATS9473": '3003',
     "ATS9625": '6',
     "ATS9626": '7',
+    "ATS9628": '1047',
     "ATS9870": '3',
     "ATS9872": '1042',
-    "ATS9628": '1047',
-    "ATS9470": '1043',
     "ATST371": '1044',
     "ATST352": '1045',
     "ATST146": '1046',
-    "ATS9182": '3007',
-    "ATS9364": '3004',
-   # "ATS9428": '3008',
-   # "ATS9442": '1048',
-   # "ATS9380": '3005',
-   # "ATS9376": '3006',
 }
 
 # Identifies a package that can be registered to the website
@@ -99,6 +104,7 @@ PackageId = namedtuple(
     [
         'key',  # type: str
         'os',  # type: str
+        'arch',  # type: str
     ])
 
 
@@ -120,6 +126,15 @@ def get_os(filename):
         return 'Linux (.rpm)'
     raise ValueError("Could not determine OS for file {}".format(filename))
 
+def get_arch(filename):
+    filename = filename.lower()
+    if 'arm64' in filename:
+        return 'arm64'
+    if 'aarch64' in filename:
+        return 'arm64'
+    else:
+        return 'x86_64'
+
 
 def get_product_key(filename):
     for key in PRODUCTS:
@@ -136,6 +151,8 @@ Package = namedtuple(
         'product_id',  # type: str
         'installer',  # type: str
         'readme',  # type: str
+        'name',  # type: str
+        'arch',  # type: str
     ])
 
 
@@ -150,22 +167,26 @@ def packages_from_files(files):
                 raise ValueError(
                     "Multiple release notes files for package {} found".format(
                         key))
-            readmes[key] = filename
+            filename = os.path.normpath(filename)
+            readmes[key] = filename.split(os.sep)[-1]
         else:
             # This is an installer
-            packageid = PackageId(key, get_os(filename))
+            packageid = PackageId(key, get_os(filename), get_arch(filename))
             if packageid in installers:
                 raise ValueError(
                     "Multiple installers files for package ID {} found.".
                     format(packageid))
-            installers[packageid] = filename
+            filename = os.path.normpath(filename)
+            installers[packageid] = filename.split(os.sep)[-1]
     packages = []  # type: List[Package]
     for packageid in installers:
         packages.append(
             Package(os=packageid.os,
                     product_id=PRODUCTS[packageid.key],
                     installer=installers[packageid],
-                    readme=readmes[packageid.key]))
+                    readme=readmes[packageid.key],
+                    name=packageid.key,
+                    arch=packageid.arch))
     return packages
 
 
